#ifndef ADuC7026_h
#define ADUC7026_h

#include <stdint.h>	//standard exact width integer types

//NOTE: all register-variables need to be VOLATILE!
//Otherwise the Program will not notice changes in the register
//which are happening from outside the code


//COMMENT: Registers are Addresses -> Pointers
//define with (*(REGxx (ADDRESS))) so that one can write directly to it without dereferencing every time!
#define REG16 (uint16_t*)
#define REG32 (uint32_t*)
#define REG8  (uint8_t*)



//---------------------------------------
//POWER-MODE AND PLL REGISTERS
//---------------------------------------

//to write to POWCON the Sequence is: POWKEY1 = 0x01; POWCON= ??; POWKEY2=0xF4;
#define POWKEY1 (*(REG32 (0xFFFF0404)))
#define POWCON  (*(REG8 (0xFFFF0408)))
#define POWKEY2 (*(REG32 (0xFFFF040C)))
	#define PKey1_Val 0x01
	#define PKey2_Val 0xF4
	#define Active_Mode (1<<6)|(1<<5)|(1<<4)
	#define CLK41MHz (1<<2)|(1<<1)|(1<<0)

//---------------------------------------
//REMAP
//---------------------------------------
#define MEM_REMAP (*(REG8 (0xFFFF0220)))
	#define Map_RAM (1<<0)

//---------------------------------------
//GENRALE PURPOSE IN/OUT REGISTERS (GPIO)
//---------------------------------------

//GPxCON: Describes the function of the Ports: 00 for GPIO, others for special functions
//see Data-Sheet ADuC7026 page 64 Table 78
//(GPIO by default)

#define GP0CON		(*(REG32 (0xFFFFF400)))
#define GP1CON		(*(REG32 (0xFFFFF404)))
#define GP2CON		(*(REG32 (0xFFFFF408)))
#define GP3CON		(*(REG32 (0xFFFFF40C)))
#define GP4CON		(*(REG32 (0xFFFFF410)))


//GPxDAT: Description of Data:  ->Bit 31:24 : direction 0 input / 1 output
//								->Bit 23:16 : Data output
//								->Bit 15:8  : State at RESET (read only)
//								->Bit 7:0   : Data Input (read only)

#define GP0DAT		(*(REG32 (0xFFFFF420)))
#define GP1DAT		(*(REG32 (0xFFFFF430)))
#define GP2DAT		(*(REG32 (0xFFFFF440)))
#define GP3DAT		(*(REG32 (0xFFFFF450)))
#define GP4DAT		(*(REG32 (0xFFFFF460)))
	#define Pin0_Dir (1<<24)	//Set for output, clear for Input
	#define Pin1_Dir (1<<25)
	#define Pin2_Dir (1<<26)
	#define Pin3_Dir (1<<27)
	#define Pin4_Dir (1<<28)
	#define Pin5_Dir (1<<29)
	#define Pin6_Dir (1<<30)
	#define Pin7_Dir (1<<31)



//GPxSET: Bit 23:16 if Set, Bit is Set. If cleared, no effect
//other Bits are reserved
//GPxClR: As GPxSET, but Clears the corresponding Bits

#define GP0SET		(*(REG32 (0xFFFFF424)))
#define GP1SET		(*(REG32 (0xFFFFF434)))
#define GP2SET		(*(REG32 (0xFFFFF444)))
#define GP3SET		(*(REG32 (0xFFFFF454)))
#define GP4SET		(*(REG32 (0xFFFFF464)))
#define GP0CLR		(*(REG32 (0xFFFFF428)))
#define GP1CLR		(*(REG32 (0xFFFFF438)))
#define GP2CLR		(*(REG32 (0xFFFFF448)))
#define GP3CLR		(*(REG32 (0xFFFFF458)))
#define GP4CLR		(*(REG32 (0xFFFFF468)))
	#define PIN0 (1<<16)	//Set for output, clear for Input
	#define PIN1 (1<<17)
	#define PIN2 (1<<18)
	#define PIN3 (1<<19)
	#define PIN4 (1<<20)
	#define PIN5 (1<<21)
	#define PIN6 (1<<22)
	#define PIN7 (1<<23)


//-----------------------------------------
//TIMER 0 REGISTERS
//-----------------------------------------

#define T0LD 	(*(REG16 (0xFFFF0300)))
#define T0VAL	(*(REG16 (0xFFFF0304)))
#define T0CON	(*(REG16 (0xFFFF0308)))
#define T0CLRI	(*(REG8  (0xFFFF030C)))

//-----------------------------------------
//TIMER 1 REGISTERS
//-----------------------------------------

#define T1LD 	(*(REG32 (0xFFFF0320)))
#define T1VAL	(*(REG32 (0xFFFF0324)))
#define T1CON	(*(REG32 (0xFFFF0328)))
#define T1CLRI	(*(REG8  (0xFFFF032C)))
#define T1CAP	(*(REG32 (0xFFFF0330)))

//------------------------------------------
//IRQ REGISTERS
//------------------------------------------

#define IRQSTA	(*(REG32 (0xFFFF0000)))
#define IRQSIG	(*(REG32 (0xFFFF0004)))
#define IRQEN	(*(REG32 (0xFFFF0008)))
#define IRQCLR	(*(REG32 (0xFFFF000C)))



#endif /* ADuC7026_h*/
